#!/bin/bash
########################################################################
# build_lhapdf.sh
########################################################################

####################################
# Useful variables.
prog=${0##*/}
thisdir=$(cd "$(dirname "${0}")" && pwd -P)
trace=maybe_xtrace # Could set to xtrace instead for unconditional.

####################################
# Useful functions.

# Report usage.
usage() {
  cat <<EOF
USAGE: ${prog} ${ssi_msg_config_usage:-<ssi-msg-config-options>} <product-dir> (e19|e20|c7):p382 debug|prof [tar]
       ${prog} -h|--help

  Build ${package} and package for UPS.

EOF
  (( ${1:-0} < 2 )) && return;
  cat <<EOF
OPTIONS

  -h|--help

     This help.

${ssi_msg_config_options:-"  <ssi-msg-config-options>

    [Supply <product-dir> to enable more details]

"}\
ARGUMENTS

  <product-dir>
    The top-level UPS directory in which the source code distribution
    should be constructed.

  (e19|e20|c7):p382
    Qualifier string.

  debug|prof
    Build type.

  [tar]
    Generate a binary installation archive.

${ssi_msg_config_environment}
ENVIRONMENT

  DEBUG_BUILD
    Full command tracing for the build (VERBOSE=1)

  DEBUG_PATCHES
  VERBOSE_PATCHES
    Command tracing and increased verbosity for source unpacking and patching
    operations. DEBUG_PATCHES will cause exit after patches have been applied.

${ssi_msg_config_environment}
EOF
}

# Local (dumb) ssi_die() function for use until we source ssi_functions.
ssi_die() {
  printf "FATAL ERROR: ${*}\n" 1>&2; exit 1
}

# Make sure we have UPS and ssibuildshims set up.
ensure_ssibuildshims() {
  if ! (( $# )) || [ "${@: -1}" = "-h" ] || [ "${@: -1}" = "--help" ]; then
    # We\'ve been asked for help: give as much as we can.
    usage 2; exit 1
  fi
  # Find <product-dir> from our arguments:
  local arg
  for arg in "${@}"; do
    [[ "$arg" ==  -* ]] && continue;
    product_dir="${arg}"; break
  done
  if [ -z "${product_dir}" ]; then
    # Bad usage.
    ssi_die "Bad <product-dir> ${product_dir}\n$(usage)"
    exit 2
  elif ! { [ -d "${product_dir}/.upsfiles" ] && [ -w "${product_dir}" ]; }; then
    # Something wrong with <product-dir>.
    ssi_die "<product-dir> not a valid, writable unified UPS directory: ${product_dir}"
  fi
  # Promising...
  local abs_product_dir="$(cd "${product_dir}" 2>/dev/null 2>&1 && pwd -P)"
  (( $? )) && ssi_die "unable to cd to <product-dir>: ${product_dir}"
  # Absolute is safer.
  product_dir="${abs_product_dir}"
  # Find our own bootstraps and yank 'em.
  local ssibuildshims_topdir="${product_dir}/ssibuildshims/${ssibuildshims_version}"
  [ -f "${ssibuildshims_topdir}/bin/ssi_functions" ] && \
    source "${ssibuildshims_topdir}/bin/ssi_functions" >/dev/null 2>&1 && \
    ensure_ups "${product_dir}" && \
    setup ssibuildshims ${ssibuildshims_version} -z "${product_dir}" || \
    ssi_die "unable to find ssibuildshims ${ssibuildshims_version} in
${product_dir} and set it up with UPS."
}

set_qualifiers() {
    qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
    for qual in "${qlist[@]}"; do
	    case $qual in
        e19|e20|c7)
        	cqual=$qual
        	;;
        p382)
        	pqual=$qual
        	;;
        *)
        	ssi_die "Unrecognized qualifier $qual"
	    esac
    done
    if [[ -z "$cqual" ]]; then
	    ssi_die "Base qualifier is mandatory"
    fi
    if [[ -z "$pqual" ]]; then
	    ssi_die "A python qualifier is mandatory (e.g. p382)."
    fi
    basequal=${cqual}:${pqual}
}

set_compiler_flags() {
  case ${extraqual} in
    debug)
      cxxflg=(-g -O0)
      ;;
    prof)
      cxxflg=(-g -fno-omit-frame-pointer -O3 -DNDEBUG)
      ;;
    opt)
      cxxflg=(-O3 -DNDEBUG)
      ;;
    * )
      echo "ERROR: please specify debug, opt, or prof"
      usage
      exit 1
      ;;
  esac
  fflg=("${cxxflg[@]}")
  case ${cqual} in
    e19|e20)
      cc=gcc
      cxx=g++
      fc=gfortran
      cxxstd=17
      ;;
    c7)
      cc=clang
      cxx=clang++
      fc=gfortran
      cxxstd=17
      ;;
    *)
      echo "ERROR: unrecognized compiler qualifier ${cqual}"
      usage
      exit 1
  esac
  cxxflg+=(-std=c++${cxxstd} -Wno-deprecated-declarations)
}

# Ascertain whether we built and installed correctly.
is_built() {
  verify_in_filesystem -d "${@}" "${pkgdir}/lib"
}

# If a distribution tarball is wanted and the installation is
# successful, then make it and exit.
maybe_make_distribution() {
  if [ "${maketar}" = "tar" ]; then
    is_built "$@" && make_basic_distribution && exit
  fi
}

########################################################################
# Main.
########################################################################

####################################
# Package name and version information.
package=lhapdf
origpkgver=v6_2_3
pkgver=${origpkgver}a
ssibuildshims_version=v2_00_00
pkgdotver=${origpkgver//_/.}; pkgdotver=${pkgdotver#v}
pkgtardir=LHAPDF-${pkgdotver}
pkgtarfile=${pkgtardir}.tar.gz

####################################
# Handle options and arguments.

# Set up ssibuildshims and UPS / product_dir as early as possible.
ensure_ssibuildshims "${@}"

# Initialization and defaults.
(( ssi_trace = 1 ))

# Parse options.
while true; do
  case $1 in
    -h|--help) usage 2; exit 1;;
    --) shift; break;;
    -*) ssi_handle_config_options "${1}" || \
      ssi_die 2 "unrecognized option $1\n$(usage)";;
    *) shift; break # Swallow <product-dir>: dealt with.
  esac
  shift
done

# Environment overrides.
ssi_honor_environment_config

##################
# Remaining arguments.
basequal=${1}; shift
extraqual=${1}; shift
maketar=${1}; shift

scriptdir=${product_dir}/${package}/${pkgver}/scripts

# Deal with qualifiers.
set_qualifiers

define_basics || exit

# Possible short-circuit.
maybe_make_distribution -q

ssi_info "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${pkgflvr})"

####################################
# Ensure directories.
mkdir -p "${pkgdir}" || \
  ssi_die "failed to create installation directory ${pkgdir}"
# Remove any existing sourcedir to avoid patch conflicts.
rm -rf "${sourcedir}" && mkdir -p "${sourcedir}" || \
  ssi_die "failed to create source directory ${sourcedir}"
# Clean build.
rm -rf "${blddir}" && mkdir -p "${blddir}" || \
  ssi_die "failed to create build directory ${blddir}"
####################################

####################################
# Declare ourselves privately, and then set ourselves up to allow setup
# of dependencies.
${trace} fake_declare_product "${ssi_args[@]}" -x \
  "${product_dir}" ${package} ${pkgver} ${fullqual} || exit
####################################

# Print active products.
echo
ups active
echo

cd "${sourcedir}" || ssi_die "unable to cd to source directory ${sourcedir}"

####################################
# Expand and patch the source.
${trace} prepare_source "${tardir}/${pkgtarfile}" \
  setup.py.in.patch fix-configure.patch || exit

# Fix includes in Python wrappers.
grep -l -Fe '../../include/' -r . | \
  xargs perl -wapi\~ -e 's&\Q../../include/\E&&g' || \
  ssi_die "unable to fix relative includes in Python wrappers"

[ -n "${DEBUG_PATCHES}" ] && exit 1
####################################

cd "${blddir}" || ssi_die "unable to cd to build directory ${blddir}"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

####################################
# Configure, build, test, and install.

set_compiler_flags

# Configure.
CC=${cc} CXX=${cxx} FC=${fc} CXXFLAGS="${cxxflg[@]}" FFLAGS="${fflg[@]}" \
  ${trace} "${sourcedir}/${pkgtardir}/configure" --prefix="${pkgdir}" \
  || ssi_die "configure failed"

# Build.
CC=${cc} CXX=${cxx} FC=${fc} CXXFLAGS="${cxxflg[@]}" FFLAGS="${fflg[@]}" \
  ${trace} make ${DEBUG_BUILD:+V=1} -j ${ncore} || ssi_die "build failed"

# Test.
CC=${cc} CXX=${cxx} FC=${fc} CXXFLAGS="${cxxflg[@]}" FFLAGS="${fflg[@]}" \
  ${trace} make check || ssi_die "tests failed"

# Install.
CC=${cc} CXX=${cxx} FC=${fc} CXXFLAGS="${cxxflg[@]}" FFLAGS="${fflg[@]}" \
  ${trace} make install || ssi_die "install failed"

####################################
# Installation verification and post-processing.

# Verify installation.
is_built -q || ssi_build_failed ${package}

ssi_info "post-processing installation"

# Install example source code by hand.
${trace} cp -pr "${sourcedir}/${pkgtardir}/examples" "${pkgdir}/examples"
${trace} rm -f ${pkgdir}/examples/Makefile.*
${trace} cp -p ${blddir}/examples/* ${pkgdir}/examples/

# Fix paths in bin/lhapdf-config.
lcfg="${pkgdir}/bin/lhapdf-config"
${trace} sed -E -i.bak -e "s%${pkgdir}%\${LHAPDF_FQ_DIR}%g" "${lcfg}" || \
  ssi_die "unable to fix lhapdf-config"

# Remove libtool files since they contain hardwired paths.
${trace} rm -vf ${LHAPDF_FQ_DIR}/lib/*.la
####################################

ssi_info "finished building ${package} ${pkgver}"

# Declare and set up the installed package.
unsetup ${package} >/dev/null 2>&1
${trace} declare_product -x "${ssi_args[@]}" \
  "${product_dir}" ${package} ${pkgver} ${fullqual} || exit

ssi_info "${package} is installed at ${pkgdir}"

# This must be last.
maybe_make_distribution
